"""
The module is there to give the possibility to access
filters without having to enter them separately
"""
EQUALS = "="
NOT_EQUALS = "!="
GREATER = ">"
GREATER_EQUALS = ">="
LESS = "<"
LESS_EQUALS = "<="
